// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "FrameOperation.h"
#include "Buffer.h"
#include "Vector.h"
#include <math.h>
//#include "window.h"
//#include "mdct.h"
#include "codec.h"
#include "psy.h"

class VorbisPsycho;

DECLARE_NODE(VorbisPsycho)
/*Node

 * @name VorbisPsycho
 * @category Signal:Audio
 * @description No description available

 * @input_name INPUT
 * @input_description No description available

 * @output_name OUTPUT
 * @output_description No description available

 * @parameter_name LENGTH
 * @parameter_description No description available

END*/



static vorbis_info_psy _psy_set0={
  1,/*athp*/
  1,/*decayp*/
  1,/*smoothp*/
  0,8,0.,

  -130.,

  1,/* tonemaskp */
  {-80.,-80.,-80.,-80.,-100.}, /* remember that el 0,2 is a 80 dB curve */
  {-35.,-40.,-60.,-80.,-80.}, /* remember that el 4 is an 80 dB curve, not 100 */
  {-35.,-40.,-60.,-80.,-100.},
  {-35.,-40.,-60.,-80.,-100.},
  {-35.,-40.,-60.,-80.,-100.},
  {-35.,-40.,-60.,-80.,-100.},
  {-35.,-40.,-60.,-80.,-100.},  

  1,/* peakattp */
  {-12.,-12.,-12.,-16.,-18.},
  {-12.,-12.,-12.,-16.,-18.},
  {-12.,-12.,-12.,-16.,-18.},
  {-12.,-12.,-12.,-16.,-18.},
  {-12.,-12.,-12.,-16.,-18.},
  {-8.,-10.,-12.,-16.,-18.},
  {-6.,-8.,-10.,-12.,-12.},

  1,/*noisemaskp */
  {-100.,-100.,-100.,-200.,-200.},
  {-100.,-100.,-100.,-200.,-200.},
  {-100.,-100.,-100.,-200.,-200.},
  {-60.,-60.,-60.,-80.,-80.},
  {-60.,-60.,-60.,-80.,-80.},
  {-60.,-60.,-60.,-80.,-80.},
  {-55.,-55.,-60.,-80.,-80.},

  100.,

  .9998, .9999  /* attack/decay control */
};




class VorbisPsycho : public FrameOperation {
   
   int inputID;
   int inputLength;
   vector<double> pcm;
   vector<double> floor;
   vector<double> decay;
   vorbis_look_psy p_look;

public:
   VorbisPsycho(string nodeName, ParameterSet params)
   : FrameOperation(nodeName, params)
   {
      inputID = addInput("INPUT");
      if (parameters.exist("INPUTLENGTH"))
         inputLength = dereference_cast<int> (parameters.get("INPUTLENGTH"));
      else inputLength = dereference_cast<int> (parameters.get("LENGTH"));

      floor.resize(inputLength);
      pcm.resize(inputLength);
      decay.resize(inputLength);
      for (int i=0;i<inputLength;i++)
	 decay[i]=0;
      inOrder = true;

   }

   ~VorbisPsycho() {}

   virtual void specificInitialize()
   {
      //window=_vorbis_window(0,inputLength*2,inputLength,inputLength);
      //mdct_init(&m_look,inputLength*2);
      _vp_psy_init(&p_look,&_psy_set0,inputLength,16000);
      
      this->FrameOperation::specificInitialize();
   }

   void calculate(int output_id, int count, Buffer &out)
   {
      NodeInput input = inputs[inputID];
      ObjectRef inputValue = input.node->getOutput(input.outputID, count);

      Vector<float> &output = object_cast<Vector<float> > (out[count]);
      if (inputValue->status != Object::valid)
      {
         output.status = inputValue->status;
         return;
      }
      const Vector<float> &in = object_cast<Vector<float> > (inputValue);
      
      for (int i=0;i<inputLength;i++)
	 pcm[i] = in[i];

      _vp_compute_mask(&p_look,&pcm[0],&floor[0],&decay[0]);
      
/*
      	for(int j=0;j<inputLength;j++)floor[j]=todB(floor[j])+150;
	double amp=_curve_to_lpc(floor.begin(),lpc.begin(),&floorlook,count);
	_lpc_to_curve(floor,lpc,sqrt(amp),&floorlook,"Ffloor",count);
	for(int j=0;j<inputLength;j++)floor[j]=fromdB(floor[j]-150);
*/
      //mdct_forward(&m_look,pcm.begin(),pcm.begin());

      for (int i=0;i<outputLength;i++)
      {
         output[i]=floor[i];
      }
      
      output.status = Object::valid;
   }

};
